@@ -269,7 +269,7 @@ findOneCenterline(bitmap_type    const bitmap,
     */
     {
         pm_pixelcoord pos;
-        pos.col = col; pos.row = bitmap.height - row - 1;
+        pos.col = originalCol; pos.row = bitmap.height - originalRow - 1;
         LOG2(" (%d,%d)", pos.col, pos.row);
         append_outline_pixel(&outline, pos);
     }
